import pytest
from downerhelper.logicapp import send_email
from downerhelper.secrets import get_key_url
import os
import base64
import logging

file_name = "temp.txt"

def test_send_email(keyvault_url):
    key_url = get_key_url(os.getenv('SEND_EMAIL_SECRET'), keyvault_url, None)
    recipients = ["marcus.oates@downergroup.com"] * 2
    subject = "Pytest Email"
    body = "This is a test email sent from a pytest"

    assert send_email(key_url, recipients, subject, body)

    with pytest.raises(Exception):
        send_email(None, recipients, subject, body)

    with pytest.raises(Exception):
        send_email(key_url, "", subject, body)

    attachments = []
    with open(file_name, 'rb') as file:
        attachments.append({
            "Name": file_name,
            "ContentBytes": base64.b64encode(file.read()).decode('utf-8')
        })
    with open(file_name, 'rb') as file:
        attachments.append({
            "Name": file_name,
            "ContentBytes": file.read()
        })

    assert send_email(key_url, recipients, subject, body, attachments=attachments)
    
    cc=bcc=recipients

    assert send_email(key_url, recipients, subject, body, cc=cc, bcc=bcc)

    recipients=cc=bcc=";".join(recipients)

    assert send_email(key_url, recipients, subject, body, cc=cc, bcc=bcc)

    for importance in ["Normal", "Low", "High"]:
        assert send_email(key_url, recipients, subject, body, importance=importance)


import requests
import json
import pytz
from datetime import datetime, timezone

def get_access_token(root, username, pwd, queue):
    try:
        tokenUrl = f"{root}/portal/sharing/rest/generateToken"
        payload  = f"username={username}&password={pwd}&expiration=60&client=referer&referer={root}&f=json"
        headers = {
            'content-type': "application/x-www-form-urlencoded",
            'accept': "application/json",
            'cache-control': "no-cache"
        }
        response = requests.post(tokenUrl, data=payload, headers=headers)

        if response.status_code != 200:
            raise Exception("Failed to get token")
        
        return response.json()['token']
    except Exception as e:
        queue.add('ERROR', f"Error getting access token: {e}")
        raise e
    
def query_feature_service(token, feature_service, where, queue, add_where=None):
    try:
        if add_where is not None:
            where = f"{where} and {add_where}"
        
        url = f"{feature_service}/query"
        headers = {
            'Accept': '*/*',
            'Accept-Encoding': 'gzip, deflate',
            'Connection': 'close',
            'Content-Type': 'application/json',
            'User-Agent': 'python-requests/3.10'
        }

        result_data = {}
        features = []

        limit = 1000
        offset = 0
        count = 0
        while count < 100:
            params  = {
                'token': token, 
                'f':'json',
                'outFields': '*',
                'where': where,
                "resultRecordCount": limit,
                "resultOffset": offset
            }
            response = requests.get(url, headers=headers, params=params)

            if response.status_code != 200:
                raise Exception("Failed to query feature service")
            
            json_data = json.loads(response.text)
            if 'status' in json_data.keys() and json_data['status'].lower() == 'error':
                if 'message' in json_data.keys():
                    raise Exception(json_data['message'])
                elif 'messages' in json_data.keys():
                    raise Exception(' '.join(json_data['messages']))
                else:
                    raise Exception(f'query returned an error: {json.dumps(json_data)}')
            elif 'features' not in json_data.keys():
                raise Exception(f'JSON data is unexpected shape: {json.dumps(json_data)}') 

            features.extend(json_data['features'])
            if len(json_data['features']) < limit: break
            if count == 0: result_data = json_data

            offset += limit
            count += 1

        result_data['features'] = features

        queue.add('INFO', f"Found {len(result_data['features'])} in the feature service")
        return result_data
    
    except Exception as e:
        queue.add('ERROR', f"Error querying feature service {feature_service}: {e}")
        raise e
    
def query_attachments(token, feature_service, object_id, queue):
    try:
        url = f"{feature_service}/queryAttachments"
        params  = {
            'token': token, 
            'f':'json',
            'objectIds': object_id,
        }
        headers = {
            'Accept': '*/*',
            'Accept-Encoding': 'gzip, deflate',
            'Connection': 'close',
            'Content-Type': 'application/json',
            'User-Agent': 'python-requests/3.10'
        }
        response = requests.get(url, headers=headers, params=params)

        if response.status_code != 200:
            raise Exception("Failed to query feature service")
        elif 'error' in response.json().keys():
            raise Exception(response.json()['error']['message'])
        
        return json.loads(response.text)
    except Exception as e:
        queue.add('ERROR', f"Error querying attachments at {feature_service} for object {object_id}: {e}")
        raise e
    
def get_attachment(token, feature_service, object_id, attachment_id, queue):
    try:
        url = f"{feature_service}/{object_id}/attachments/{attachment_id}"
        params  = {
            'token': token, 
        }
        headers = {
            'Accept': '*/*',
            'Accept-Encoding': 'gzip, deflate',
            'Connection': 'close',
            'Content-Type': 'application/json',
            'User-Agent': 'python-requests/3.10'
        }
        response = requests.get(url, headers=headers, params=params)

        if response.status_code != 200 or response.content in [None, '']:
            raise Exception("Failed to get attachment")
        
        return response.content
    except Exception as e:
        queue.add('ERROR', f"Error getting attachment at {feature_service} for object {object_id} and {attachment_id}: {e}")
        raise e
    
def update_feature_service(token, feature_service, features, queue):
    try:
        url = f"{feature_service}/updateFeatures"
        params  = {
            'token': token, 
            'f':'json',
            'features': json.dumps(features),
        }
        headers = {
            'Accept': '*/*',
            'Accept-Encoding': 'gzip, deflate',
            'Connection': 'close',
            'Content-Type': 'application/json',
            'User-Agent': 'python-requests/3.10'
        }
        response = requests.post(url, headers=headers, params=params)

        if not response.ok or 'error' in response.json().keys(): 
            raise Exception("Failed to update feature service")
        
        for result in response.json()['updateResults']:
            if result['success']: continue
            raise Exception(result['error']['description'])
        
        return True
    except Exception as e:
        queue.add('ERROR', f"Error updating feature service at {feature_service}: {e}")
        raise e
    
def timestamp_to_datetime(timestamp, queue, tz_str='Pacific/Auckland'):
    try:
        if isinstance(timestamp, str): timestamp = int(timestamp)
        tz = pytz.timezone(tz_str)
        return datetime.fromtimestamp(timestamp / 1000, tz=timezone.utc).astimezone(tz).isoformat()
    except Exception as e:
        queue.add("ERROR", f"Error converting timestamp {timestamp} to datetime: {e}")
        raise e


import logging
import psycopg2 as pg
from psycopg2 import sql
from datetime import datetime, timezone
from downerhelper.secrets import get_config_dict
import requests
from downerhelper.logicapp import send_email

class PostgresLogQueue():
    def __init__(self, logger_name, job_id, table, db_config, print_logs=False):
        try:
            if '' in [logger_name, job_id, table] or \
                None in [logger_name, job_id, table] or db_config == {}:
                raise Exception("Invalid parameters")
            self.logger_name = logger_name
            self.job_id = job_id
            self.db_config = db_config
            self.table = table
            self.print_logs = print_logs
            self.queue = [
                {
                    'levelname': 'INFO',
                    'message': f'queue: {logger_name} created for job_id: {job_id}',
                    'created_at': datetime.now(timezone.utc)
                }
            ]
        except Exception as e:
            logging.error(f"Error setting up PostgresLogHandler: {e}")
            raise e

    def add(self, levelname, message):
        self.queue.append({
            'levelname': levelname,
            'message': message,
            'created_at': datetime.now(timezone.utc)
        })
        if not self.print_logs: return
        
        if levelname == 'ERROR':
            logging.error(message)
        elif levelname == 'WARNING':
            logging.warning(message)
        elif levelname == 'DEBUG':
            logging.debug(message)
        else:
            logging.info(message)

    def save(self):
        try:
            conn = pg.connect(**self.db_config)
            cursor = conn.cursor()
            cursor.execute("set time zone 'UTC'")
            cursor.execute(f"""
            create table if not exists {self.table} (
                id serial primary key,
                created_at timestamptz default now(),
                name varchar(255),
                levelname varchar(50),
                message text,
                job_id varchar(255) not null,
                is_checked boolean default false
            )""")
            conn.commit()
            
            for log in self.queue:
                cursor.execute(f"""
                insert into {self.table} (name, levelname, message, job_id, created_at)
                values (%s, %s, %s, %s, %s)
                """, (self.logger_name, log['levelname'], log['message'], self.job_id, log['created_at']))
            conn.commit()

            if cursor: cursor.close()
            if conn: conn.close()

            self.queue = []

        except Exception as e:
            logging.error(f"Error saving logs: {e}")
            if cursor: cursor.close()
            if conn: conn.close()

    def check_logs(self, key_url, recipients, interval_hours=24):
        try:
            conn = cur = None
            conn = pg.connect(**self.db_config)
            cur = conn.cursor()

            cur.execute(sql.SQL("""
                select name, levelname, message, job_id, id
                from {}
                where levelname != 'INFO'
                and is_checked = false
                and created_at > now() - interval {}::interval;
            """).format(
                sql.Identifier(self.table),
                sql.Literal(f'{interval_hours} hours')
            ))
            rows = cur.fetchall()

            if len(rows) == 0:
                self.add('INFO', 'No errenous logs')
                self.save()
                return

            data = {}
            ids = []
            for row in rows:
                key = f"{row[0]}:{row[3]}"
                if key not in data.keys(): data[key] = []
                data[key].append({
                    'levelname': row[1],
                    'message': row[2]
                })
                ids.append(row[4])
            flows_in_error = set([k.split(':')[0] for k in data.keys()])

            table_html = """
            <table border="1">
                <tr>
                    <th>Name</th>
                    <th>Level</th>
                    <th>Message</th>
                    <th>Job ID</th>
                </tr>
            """
            for row in rows:
                table_html += f"""
                <tr>
                    <td>{row[0]}</td>
                    <td>{row[1]}</td>
                    <td>{row[2]}</td>
                    <td>{row[3]}</td>
                </tr>
                """

            subject = f"Log Check: {self.table}"
            body = f"""Flows in error: {', '.join(flows_in_error)}<br><br>{table_html}"""
            send_email(key_url, recipients, subject, body)

            cur.execute(sql.SQL("""
                update {}
                set is_checked = true
                where id = any(%s);
            """).format(
                sql.Identifier(self.table)
            ), (ids,))
            conn.commit()

        except Exception as e:
            self.add('ERROR', f"Error checking logs: {e}")
        finally:
            if cur: cur.close()
            if conn: conn.close()
            self.save()


def setup_queue(logger_name, job_id, table, db_config_name, keyvault_url, print_logs=False):
    try:
        db_config = get_config_dict(db_config_name, keyvault_url)
        return PostgresLogQueue(logger_name, job_id, table, db_config, print_logs)
    except Exception as e:
        logging.error(f"Error setting up logger: {e}")
        raise Exception("Error setting up logger")